//
// Copyright(C) 1996 Sarrus Software, Inc.
// All rights reserved.
//
// PMIAdmin.m
// 
// Simple PMIAPI program to perform administrative tasks on PMI calendars
// The program allows the creation of new calendars
// setting of access levels, group list and ownership of calendars.
// The program can read in 1.2 and 2.0 calendars but will only write out
// 2.0 format calendars.
//
#import <foundation/NSUtilities.h>
#import <bsd/sys/stat.h>

#import "PMIAdmin.h"

@implementation PMIAdmin

- initWithArgs:(char **)args count:(int)argCount
{
	[super init];

	cmdProcessor = [[SRCommandProcessor alloc] init];
	[self setupCommands];
	[cmdProcessor parseCommandLine:args count:argCount];

	return self;
}

- (void)dealloc
{
	[cmdProcessor release];	
	[super dealloc];
}

- (void)setupCommands
{
	[cmdProcessor addCommand:@"r" description:@"-r <calendarname>"
   		target:self action:@selector(readCalendar:) 
		required:YES minArgs:1];
	[cmdProcessor addCommand:@"w" description:@"-w <calendarname>"
   		target:self action:@selector(writeCalendar:) 
		required:YES minArgs:0];
	[cmdProcessor addCommand:@"ga" 
		description:@"-ga <calendarname-1> <calendarname-2>... <calendarname-n>"
   		target:self action:@selector(addToGroup:) 
		required:NO minArgs:1];
	[cmdProcessor addCommand:@"gd" 
		description:@"-gd <calendarname-1> <calendarname-2>... <calendarname-n>"
   		target:self action:@selector(removeFromGroup:) 
		required:NO minArgs:1];
	[cmdProcessor addCommand:@"a" 
		description:@"-a <username> <access_level>"
   		target:self action:@selector(setAccessLevel:) 
		required:NO minArgs:2];
	[cmdProcessor addCommand:@"o" 
		description:@"-o <username>"
   		target:self action:@selector(setOwner:) 
		required:NO minArgs:1];
}

- (void)run
{
	if ([cmdProcessor checkCommands])
	{
		[cmdProcessor runCommand:@"r"];
		[cmdProcessor runOptionalCommands];
		[cmdProcessor runCommand:@"w"];
	}
	else [self usage];
}

- (void)readCalendar:(NSArray *)args
{
	struct stat stbuf;

	if (!calendar)
	{
		NSString *calendarName=[args objectAtIndex:0];

		if (stat([calendarName cString], &stbuf) >= 0)
		{
			calendar = [[PMICalendar calendarFromFile:calendarName] retain];
		}
		else
		{
			calendar = [[PMICalendar calendar] retain];
			[calendar setCalendarFilename:calendarName];
		}
	}
}

- (void)writeCalendar:(NSArray *)args
{
	if (calendar) 
	{
		if ([args count] > 0) [calendar saveAs:[args objectAtIndex:0]];
		else [calendar save];
	}
}

- (void)addToGroup:(NSArray *)args
{
	if (calendar)
	{
		NSEnumerator *userEnum=[args objectEnumerator];
		NSString *username;

		while (username = [userEnum nextObject])
		{
			[calendar addToGroupList:username];
		}
	}
}

- (void)removeFromGroup:(NSArray *)args
{
	if (calendar)
	{
		NSEnumerator *userEnum=[args objectEnumerator];
		NSString *username;

		while (username = [userEnum nextObject])
		{
			[calendar removeFromGroupList:username];
		}
	}
}

- (void)setAccessLevel:(NSArray *)args
{
	if (calendar)
	{
		NSString *username, *access;
		PMIAccessLevel accessLevel;
		NSEnumerator *accessEnum=[args objectEnumerator];

		while (username = [accessEnum nextObject])
		{			
			access = [accessEnum nextObject];
			if (access)	
			{
				accessLevel = (PMIAccessLevel)[access intValue];
				[calendar setAccessFor:username to:accessLevel];
			}
		}
	}
}

- (void)setOwner:(NSArray *)args
{
	if (calendar) [calendar setOwner:[args objectAtIndex:0]];
}

- (void)usage
{
	fprintf(stderr, "\nUsage:%s -r <calendarname>\n"
		"[-ga <calendarname-1> ... <calendarname-n>]\n"
		"[-gd <calendarname-1> ... <calendarname-n>]\n"
		"[-a <username-1> <access-level> ... <username-n> <access-level>]\n"
		"[-o <username>]\n"
		"-w [<calendarname>]\n", 
		[[cmdProcessor programName] cString]);
	exit(-1);
}

@end











