//
// Copyright(C) 1996 Sarrus Software, Inc.
// All rights reserved.
//
// SRCommandProcessor.m
//

#import <foundation/NSUtilities.h>
#import <foundation/NSPathUtilities.h>

#import "SRCommandProcessor.h"

@implementation SRCommandProcessor

+ commandProcessor
{
	return [[[self alloc] init] autorelease];
}

- init
{
	[super init];

	allowableCommands = [[NSMutableDictionary dictionary] retain];
	parsedCommands = [[NSMutableDictionary dictionary] retain];

	commandPrefix = @"-";
	[commandPrefix retain];
	prefixLength = [commandPrefix length];

	return self;
}

- (void)dealloc
{
	[commandPrefix release];
	[programName release];
	[allowableCommands release];
	[parsedCommands release];
	[super dealloc];
}

- (void)addCommand:(SRCommand *)command
{
	SRCommand *ac=[allowableCommands objectForKey:[command name]];
	SRCommand *pc=[parsedCommands objectForKey:[command name]];
	
	if (ac == nil && pc == nil) 
	{
		[allowableCommands setObject:command forKey:[command name]];
	}
}

- (void)addCommand:(NSString *)cmdName
	description:(NSString *)cmdDesc
	target:aTarget
	action:(SEL)anAction
	required:(BOOL)isRequired
	minArgs:(unsigned int)minNumberArgs
{
	SRCommand *ac=[allowableCommands objectForKey:cmdName];
	SRCommand *pc=[parsedCommands objectForKey:cmdName];
	
	if (ac == nil && pc == nil) 
	{
		SRCommand *cmd=[[SRCommand alloc] initCommandNamed:cmdName];

		[cmd setDescription:cmdDesc];
		[cmd setTarget:aTarget action:anAction];
		[cmd setRequired:isRequired];
		[cmd setMinimumNumberArguments:minNumberArgs];

		[allowableCommands setObject:cmd forKey:cmdName];
	}
}

- (void)removeCommand:(NSString *)name
{
	[allowableCommands removeObjectForKey:name];
	[parsedCommands removeObjectForKey:name];
}

- (void)setCommandPrefix:(NSString *)prefix
{
	[commandPrefix release];
	commandPrefix = [[NSString alloc] initWithString:prefix];
	prefixLength = [commandPrefix length];
}

- (NSString *)commandPrefix
{
	return commandPrefix;
}

- (void)parseCommandLine:(char **)args count:(int)argCount
{
	int i=1;	// skip past name of program

	NSString *fullProgramName=[NSString stringWithCString:args[0]];
	programName = [[NSString alloc] initWithString:
					[fullProgramName lastPathComponent]];


	[parsedCommands removeAllObjects];
	while (i < argCount)
	{
		NSString *fullCommandName=[NSString stringWithCString:args[i]];

		if ([fullCommandName hasPrefix:commandPrefix])
		{
			SRCommand *cmd=[allowableCommands objectForKey:
						[fullCommandName substringFromIndex:prefixLength]];

			while (++i < argCount && [[NSString stringWithCString:args[i]] 
					hasPrefix:commandPrefix] == NO)
			{
				[cmd addArgument:[NSString stringWithCString:args[i]]];
			}

			if (cmd) 
			{
				cmd = [cmd retain];
				[allowableCommands removeObjectForKey:[cmd name]];
				[parsedCommands setObject:cmd forKey:[cmd name]];
			}
		}
		else i++;
	}
}

- (void)runOptionalCommands
{
	NSEnumerator *cmdEnum;
	SRCommand *cmd;

	cmdEnum = [parsedCommands objectEnumerator];
	while (cmd = [cmdEnum nextObject]) 
	{
		if ([cmd required] == NO) 
		{
			[cmd run];
			[parsedCommands removeObjectForKey:[cmd name]];
		}
	}
}

- (void)runCommand:(NSString *)cmdName
{
	SRCommand *cmd=[parsedCommands objectForKey:cmdName];
	
	[cmd run];
	[parsedCommands removeObjectForKey:cmdName];
}

- (BOOL)checkCommands
{
	// need to verify that all required commands are present
	// need also to verify that all commands have minimum required
	// number of arguments
	NSEnumerator *allowableEnum=[allowableCommands objectEnumerator];
	NSEnumerator *parsedEnum=[parsedCommands objectEnumerator];
	SRCommand *cmd;
	BOOL retValue=YES;

	while (cmd = [allowableEnum nextObject])
	{
		if ([cmd required] == YES)
		{
			NSString *errorString = [NSString stringWithFormat: 
				@"[%@] Command %@%@ is required. Expected %@",
				programName, commandPrefix, [cmd name], [cmd description]];

			NSLog(errorString);
			retValue = NO;
		}
	}

	while (cmd = [parsedEnum nextObject])
	{
		if ([cmd minimumNumberArguments] > [[cmd arguments] count])
		{
			NSString *errorString = [NSString stringWithFormat: 
				@"[%@] Missing argument to %@%@ command. Expected %@",
				programName, commandPrefix, [cmd name], [cmd description]];

			NSLog(errorString);
			retValue = NO;
		}
	}
	
	return retValue;
}

- (NSString *)programName
{
	return programName;
}

@end

// SRCommand

@implementation SRCommand

+ commandNamed:(NSString *)aName
{
	return [[[self alloc] initCommandNamed:aName] autorelease];
}

- initCommandNamed:(NSString *)commandName
{
	[super init];
	
	name = [[NSString alloc] initWithString:commandName];
	target = nil;
	action = NULL;	
	args = [[NSMutableArray alloc] init];
	required = NO;
	minArgs = 1;

	return self;
}

- (void)dealloc
{
	[name release];
	[description release];
	[args release];
	[super dealloc];
}

- (NSString *)name
{
	return name;
}

- (void)setName:(NSString *)newName
{
	[name release];
	name = [[NSString alloc] initWithString:newName];
}

- (void)setRequired:(BOOL)req
{
	required = req;
}

- (BOOL)required
{
	return required;
}

- (void)setDescription:(NSString *)commandDescription
{
	[description release];
	description = [[NSString alloc] initWithString:commandDescription];
}

- (NSString *)description
{
	return description;
}

- (void)setTarget:(NSObject *)someObject action:(SEL)aSelector
{
	target = someObject;
	action = aSelector;
}

- target
{
	return target;
}

- (SEL)action
{
	return action;
}

- (void)setMinimumNumberArguments:(unsigned int)numArgs
{
	minArgs = numArgs;
}

- (unsigned int)minimumNumberArguments
{
	return minArgs;
}

- (void)addArgument:(NSString *)arg
{
	if ([args containsObject:arg] == NO)
	{
		[args addObject:arg];
	}
}

- (void)removedArgument:(NSString *)arg
{
	if ([args containsObject:arg] == YES)
	{
		[args removeObject:arg];
	}
}

- (NSArray *)arguments
{
	return args;
}

- (void)run
{
	if ([target respondsTo:action])
	{
		[target perform:action with:args];
	}
}

@end



